"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __await = (this && this.__await) || function (v) { return this instanceof __await ? (this.v = v, this) : new __await(v); }
var __asyncGenerator = (this && this.__asyncGenerator) || function (thisArg, _arguments, generator) {
    if (!Symbol.asyncIterator) throw new TypeError("Symbol.asyncIterator is not defined.");
    var g = generator.apply(thisArg, _arguments || []), i, q = [];
    return i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i;
    function verb(n) { if (g[n]) i[n] = function (v) { return new Promise(function (a, b) { q.push([n, v, a, b]) > 1 || resume(n, v); }); }; }
    function resume(n, v) { try { step(g[n](v)); } catch (e) { settle(q[0][3], e); } }
    function step(r) { r.value instanceof __await ? Promise.resolve(r.value.v).then(fulfill, reject) : settle(q[0][2], r); }
    function fulfill(value) { resume("next", value); }
    function reject(value) { resume("throw", value); }
    function settle(f, v) { if (f(v), q.shift(), q.length) resume(q[0][0], q[0][1]); }
};
Object.defineProperty(exports, "__esModule", { value: true });
const decomposedMetadataTransformer_1 = require("./transformers/decomposedMetadataTransformer");
/**
 * Manages a "global" state over the course of a single metadata conversion call.
 */
class ConvertTransaction {
    constructor() {
        this.state = { recompose: {} };
        this.finalizers = new Map();
    }
    addFinalizer(finalizerCtor) {
        if (!this.finalizers.has(finalizerCtor.name)) {
            this.finalizers.set(finalizerCtor.name, new finalizerCtor());
        }
    }
    /**
     * Call right before the end of a conversion pipeline to execute logic with
     * the transaction state.
     */
    executeFinalizers() {
        return __asyncGenerator(this, arguments, function* executeFinalizers_1() {
            for (const finalizer of this.finalizers.values()) {
                yield yield __await(yield __await(finalizer.finalize(this.state)));
            }
        });
    }
}
exports.ConvertTransaction = ConvertTransaction;
class RecompositionFinalizer {
    finalize(state) {
        return __awaiter(this, void 0, void 0, function* () {
            const writerData = [];
            for (const parentName of Object.keys(state.recompose)) {
                const parentComponent = state.recompose[parentName].component;
                // only recompose children stored in transaction state
                const children = state.recompose[parentName].children;
                const recomposedXmlObj = yield decomposedMetadataTransformer_1.DecomposedMetadataTransformer.recompose(children);
                const writerFormat = decomposedMetadataTransformer_1.DecomposedMetadataTransformer.createWriterFormat(parentComponent, recomposedXmlObj);
                writerData.push(writerFormat);
            }
            return writerData;
        });
    }
}
exports.RecompositionFinalizer = RecompositionFinalizer;
//# sourceMappingURL=convertTransaction.js.map