"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const baseMetadataTransformer_1 = require("./baseMetadataTransformer");
const convertTransaction_1 = require("../convertTransaction");
const metadata_registry_1 = require("../../metadata-registry");
const streams_1 = require("../streams");
const path_1 = require("path");
const common_1 = require("../../common");
const collections_1 = require("../../collections");
class DecomposedMetadataTransformer extends baseMetadataTransformer_1.BaseMetadataTransformer {
    constructor(registry = new metadata_registry_1.RegistryAccess(), convertTransaction = new convertTransaction_1.ConvertTransaction()) {
        super(registry, convertTransaction);
        this.convertTransaction.addFinalizer(convertTransaction_1.RecompositionFinalizer);
    }
    static recompose(children, baseXmlObj = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            for (const child of children) {
                const { directoryName: groupNode } = child.type;
                const { name: parentName } = child.parent.type;
                const childContents = (yield child.parseXml())[child.type.name];
                if (!baseXmlObj[parentName]) {
                    baseXmlObj[parentName] = { '@_xmlns': common_1.XML_NS_URL };
                }
                if (!baseXmlObj[parentName][groupNode]) {
                    baseXmlObj[parentName][groupNode] = [];
                }
                baseXmlObj[parentName][groupNode].push(childContents);
            }
            return baseXmlObj;
        });
    }
    static createWriterFormat(trigger, xmlObject) {
        return {
            component: trigger,
            writeInfos: [
                {
                    source: new streams_1.JsToXml(xmlObject),
                    output: path_1.join(trigger.type.directoryName, `${trigger.fullName}.${trigger.type.suffix}`),
                },
            ],
        };
    }
    toMetadataFormat(component) {
        return __awaiter(this, void 0, void 0, function* () {
            if (component.parent) {
                const { state } = this.convertTransaction;
                const { fullName: parentName } = component.parent;
                if (!state.recompose[parentName]) {
                    state.recompose[parentName] = {
                        component: component.parent,
                        children: [],
                    };
                }
                state.recompose[parentName].children.push(component);
                // noop since the finalizer will push the writes to the component writer
                return { component: component, writeInfos: [] };
            }
            const recomposedXmlObj = yield DecomposedMetadataTransformer.recompose(component.getChildren(), (yield component.parseXml()));
            return DecomposedMetadataTransformer.createWriterFormat(component, recomposedXmlObj);
        });
    }
    toSourceFormat(component, mergeWith) {
        var _a, _b, _c, _d;
        return __awaiter(this, void 0, void 0, function* () {
            const writeInfos = [];
            const { type, fullName: parentFullName } = component;
            const parentXmlObject = { [type.name]: { [common_1.XML_NS_KEY]: common_1.XML_NS_URL } };
            let createParentXml = false;
            const rootPackagePath = component.getPackageRelativePath(parentFullName, 'source');
            const childComponentMergeSet = mergeWith
                ? new collections_1.ComponentSet(mergeWith.getChildren())
                : undefined;
            const composedMetadata = yield this.getComposedMetadataEntries(component);
            for (const [tagKey, tagValue] of composedMetadata) {
                const childTypeId = (_a = type.children) === null || _a === void 0 ? void 0 : _a.directories[tagKey];
                if (childTypeId) {
                    const childType = type.children.types[childTypeId];
                    const tagValues = Array.isArray(tagValue) ? tagValue : [tagValue];
                    for (const value of tagValues) {
                        const entryName = (value.fullName || value.name);
                        const mergeChild = (_b = childComponentMergeSet) === null || _b === void 0 ? void 0 : _b.getSourceComponents({
                            fullName: `${parentFullName}.${entryName}`,
                            type: childType,
                        }).next().value;
                        const output = ((_c = mergeChild) === null || _c === void 0 ? void 0 : _c.xml) ||
                            path_1.join(rootPackagePath, this.getOutputPathForEntry(entryName, childType, component));
                        writeInfos.push({
                            source: new streams_1.JsToXml({
                                [childType.name]: Object.assign({ [common_1.XML_NS_KEY]: common_1.XML_NS_URL }, value),
                            }),
                            output,
                        });
                    }
                }
                else {
                    // tag entry isn't a child type, so add it to the parent xml
                    if (tagKey !== common_1.XML_NS_KEY) {
                        createParentXml = true;
                    }
                    parentXmlObject[type.name][tagKey] = tagValue;
                }
            }
            if (createParentXml) {
                const parentOutput = ((_d = mergeWith) === null || _d === void 0 ? void 0 : _d.xml) ||
                    path_1.join(rootPackagePath, `${parentFullName}.${type.suffix}${common_1.META_XML_SUFFIX}`);
                writeInfos.push({
                    source: new streams_1.JsToXml(parentXmlObject),
                    output: parentOutput,
                });
            }
            return { component, writeInfos };
        });
    }
    getComposedMetadataEntries(component) {
        return __awaiter(this, void 0, void 0, function* () {
            const composedMetadata = (yield component.parseXml())[component.type.name];
            return Object.entries(composedMetadata);
        });
    }
    getOutputPathForEntry(entryName, entryType, component) {
        let output = `${entryName}.${entryType.suffix}${common_1.META_XML_SUFFIX}`;
        if (component.type.strategies.decomposition === "folderPerType" /* FolderPerType */) {
            output = path_1.join(entryType.directoryName, output);
        }
        return output;
    }
}
exports.DecomposedMetadataTransformer = DecomposedMetadataTransformer;
//# sourceMappingURL=decomposedMetadataTransformer.js.map